==> mongodb-7.0.18.gh.tar.gz: mongo-r7.0.18/src/mongo/db/s/metrics/phase_duration.h <==
/**
 *    Copyright (C) 2023-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/util/clock_source.h"
#include "mongo/util/time_support.h"

namespace mongo {

class PhaseDuration {
public:
    PhaseDuration();
    boost::optional<Date_t> getStart() const;
    boost::optional<Date_t> getEnd() const;
    void setStart(Date_t date);
    void setEnd(Date_t date);

    template <typename TimeUnit>
    boost::optional<TimeUnit> getElapsed(ClockSource* clock) const {
        auto elapsed = getElapsedMs(clock);
        if (!elapsed) {
            return boost::none;
        }
        return duration_cast<TimeUnit>(*elapsed);
    }

private:
    boost::optional<Milliseconds> getElapsedMs(ClockSource* clock) const;

    AtomicWord<Date_t> _start;
    AtomicWord<Date_t> _end;
};

}  // namespace mongo

==> mongodb-8.0.12.gh.tar.gz: mongo-r8.0.12/src/mongo/db/s/metrics/phase_duration.h <==
/**
 *    Copyright (C) 2023-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include <boost/none.hpp>
#include <boost/optional/optional.hpp>

#include "mongo/platform/atomic_word.h"
#include "mongo/util/clock_source.h"
#include "mongo/util/duration.h"
#include "mongo/util/time_support.h"

namespace mongo {

class PhaseDuration {
public:
    PhaseDuration();
    boost::optional<Date_t> getStart() const;
    boost::optional<Date_t> getEnd() const;
    void setStart(Date_t date);
    void setEnd(Date_t date);

    template <typename TimeUnit>
    boost::optional<TimeUnit> getElapsed(ClockSource* clock) const {
        auto elapsed = getElapsedMs(clock);
        if (!elapsed) {
            return boost::none;
        }
        return duration_cast<TimeUnit>(*elapsed);
    }

private:
    boost::optional<Milliseconds> getElapsedMs(ClockSource* clock) const;

    AtomicWord<Date_t> _start;
    AtomicWord<Date_t> _end;
};

}  // namespace mongo
